@@ -23,10 +23,11 @@ typedef struct State {
 } State;
 
 static void handle_self_connection_status(
-    Tox *tox, Tox_Connection connection_status, void *user_data)
+    const Tox_Event_Self_Connection_Status *event, void *user_data)
 {
     const AutoTox *autotox = (AutoTox *)user_data;
 
+    const Tox_Connection connection_status = tox_event_self_connection_status_get_connection_status(event);
     if (connection_status != TOX_CONNECTION_NONE) {
         printf("tox #%u: is now connected\n", autotox->index);
     } else {
@@ -35,10 +36,13 @@ static void handle_self_connection_status(
 }
 
 static void handle_friend_connection_status(
-    Tox *tox, uint32_t friendnumber, Tox_Connection connection_status, void *user_data)
+    const Tox_Event_Friend_Connection_Status *event, void *user_data)
 {
     const AutoTox *autotox = (AutoTox *)user_data;
 
+    const uint32_t friendnumber = tox_event_friend_connection_status_get_friend_number(event);
+    const Tox_Connection connection_status = tox_event_friend_connection_status_get_connection_status(event);
+
     if (connection_status != TOX_CONNECTION_NONE) {
         printf("tox #%u: is now connected to friend %u\n", autotox->index, friendnumber);
     } else {
@@ -70,28 +74,33 @@ static void audio_callback(void *tox, uint32_t groupnumber, uint32_t peernumber,
 }
 
 static void handle_conference_invite(
-    Tox *tox, uint32_t friendnumber, Tox_Conference_Type type,
-    const uint8_t *data, size_t length, void *user_data)
+    const Tox_Event_Conference_Invite *event, void *user_data)
 {
     const AutoTox *autotox = (AutoTox *)user_data;
+
+    const uint32_t friend_number = tox_event_conference_invite_get_friend_number(event);
+    const Tox_Conference_Type type = tox_event_conference_invite_get_type(event);
+    const uint8_t *cookie = tox_event_conference_invite_get_cookie(event);
+    const size_t length = tox_event_conference_invite_get_cookie_length(event);
+
     ck_assert_msg(type == TOX_CONFERENCE_TYPE_AV, "tox #%u: wrong conference type: %d", autotox->index, type);
 
-    ck_assert_msg(toxav_join_av_groupchat(tox, friendnumber, data, length, audio_callback, user_data) == 0,
+    ck_assert_msg(toxav_join_av_groupchat(autotox->tox, friend_number, cookie, length, audio_callback, user_data) == 0,
                   "tox #%u: failed to join group", autotox->index);
 }
 
 static void handle_conference_connected(
-    Tox *tox, uint32_t conference_number, void *user_data)
+    const Tox_Event_Conference_Connected *event, void *user_data)
 {
     const AutoTox *autotox = (AutoTox *)user_data;
     State *state = (State *)autotox->state;
 
-    if (state->invited_next || tox_self_get_friend_list_size(tox) <= 1) {
+    if (state->invited_next || tox_self_get_friend_list_size(autotox->tox) <= 1) {
         return;
     }
 
     Tox_Err_Conference_Invite err;
-    tox_conference_invite(tox, 1, 0, &err);
+    tox_conference_invite(autotox->tox, 1, 0, &err);
     ck_assert_msg(err == TOX_ERR_CONFERENCE_INVITE_OK, "tox #%u failed to invite next friend: err = %d", autotox->index,
                   err);
     printf("tox #%u: invited next friend\n", autotox->index);
@@ -99,7 +108,7 @@ static void handle_conference_connected(
 }
 
 static bool toxes_are_disconnected_from_group(uint32_t tox_count, AutoTox *autotoxes,
-        bool *disconnected)
+        const bool *disconnected)
 {
     uint32_t num_disconnected = 0;
 
@@ -138,7 +147,10 @@ static void disconnect_toxes(uint32_t tox_count, AutoTox *autotoxes,
         do {
             for (uint32_t i = 0; i < tox_count; ++i) {
                 if (!disconnect_now[i]) {
-                    tox_iterate(autotoxes[i].tox, &autotoxes[i]);
+                    Tox_Err_Events_Iterate err;
+                    Tox_Events *events = tox_events_iterate(autotoxes[i].tox, true, &err);
+                    tox_dispatch_invoke(autotoxes[i].dispatch, events, &autotoxes[i]);
+                    tox_events_free(events);
                     autotoxes[i].clock += 1000;
                 }
             }
@@ -165,7 +177,7 @@ static bool all_connected_to_group(uint32_t tox_count, AutoTox *autotoxes)
  * returns a random index at which a list of booleans is false
  * (some such index is required to exist)
  */
-static uint32_t random_false_index(const Random *rng, bool *list, const uint32_t length)
+static uint32_t random_false_index(const Random *rng, const bool *list, const uint32_t length)
 {
     uint32_t index;
 
@@ -185,7 +197,7 @@ static bool all_got_audio(AutoTox *autotoxes, const bool *disabled)
     }
 
     for (uint32_t i = 0; i < NUM_AV_GROUP_TOX; ++i) {
-        State *state = (State *)autotoxes[i].state;
+        const State *state = (const State *)autotoxes[i].state;
 
         if (disabled[i] ^ (state->received_audio_num
                            != NUM_AV_GROUP_TOX - num_disabled - 1)) {
@@ -217,7 +229,7 @@ static bool test_audio(AutoTox *autotoxes, const bool *disabled, bool quiet)
         printf("testing sending and receiving audio\n");
     }
 
-    const int16_t PCM[GROUP_AV_TEST_SAMPLES] = {0};
+    const int16_t pcm[GROUP_AV_TEST_SAMPLES] = {0};
 
     reset_received_audio(autotoxes);
 
@@ -227,7 +239,7 @@ static bool test_audio(AutoTox *autotoxes, const bool *disabled, bool quiet)
                 continue;
             }
 
-            if (toxav_group_send_audio(autotoxes[i].tox, 0, PCM, GROUP_AV_TEST_SAMPLES, 1, 48000) != 0) {
+            if (toxav_group_send_audio(autotoxes[i].tox, 0, pcm, GROUP_AV_TEST_SAMPLES, 1, 48000) != 0) {
                 if (!quiet) {
                     ck_abort_msg("#%u failed to send audio", autotoxes[i].index);
                 }
@@ -255,8 +267,12 @@ static void test_eventual_audio(AutoTox *autotoxes, const bool *disabled, uint64
     uint64_t start = autotoxes[0].clock;
 
     while (autotoxes[0].clock < start + timeout) {
-        if (test_audio(autotoxes, disabled, true)
-                && test_audio(autotoxes, disabled, true)) {
+        if (!test_audio(autotoxes, disabled, true)) {
+            continue;
+        }
+
+        // It needs to succeed twice in a row for the test to pass.
+        if (test_audio(autotoxes, disabled, true)) {
             printf("audio test successful after %d seconds\n", (int)((autotoxes[0].clock - start) / 1000));
             return;
         }
@@ -268,12 +284,12 @@ static void test_eventual_audio(AutoTox *autotoxes, const bool *disabled, uint64
 
 static void do_audio(AutoTox *autotoxes, uint32_t iterations)
 {
-    const int16_t PCM[GROUP_AV_TEST_SAMPLES] = {0};
+    const int16_t pcm[GROUP_AV_TEST_SAMPLES] = {0};
     printf("running audio for %u iterations\n", iterations);
 
     for (uint32_t f = 0; f < iterations; ++f) {
         for (uint32_t i = 0; i < NUM_AV_GROUP_TOX; ++i) {
-            ck_assert_msg(toxav_group_send_audio(autotoxes[i].tox, 0, PCM, GROUP_AV_TEST_SAMPLES, 1, 48000) == 0,
+            ck_assert_msg(toxav_group_send_audio(autotoxes[i].tox, 0, pcm, GROUP_AV_TEST_SAMPLES, 1, 48000) == 0,
                           "#%u failed to send audio", autotoxes[i].index);
             iterate_all_wait(autotoxes, NUM_AV_GROUP_TOX, ITERATION_INTERVAL);
         }
@@ -287,7 +303,7 @@ static void do_audio(AutoTox *autotoxes, uint32_t iterations)
 
 static void run_conference_tests(AutoTox *autotoxes)
 {
-    const Random *rng = system_random();
+    const Random *rng = os_random();
     ck_assert(rng != nullptr);
     bool disabled[NUM_AV_GROUP_TOX] = {0};
 
@@ -398,10 +414,10 @@ static void test_groupav(AutoTox *autotoxes)
     const time_t test_start_time = time(nullptr);
 
     for (uint32_t i = 0; i < NUM_AV_GROUP_TOX; ++i) {
-        tox_callback_self_connection_status(autotoxes[i].tox, &handle_self_connection_status);
-        tox_callback_friend_connection_status(autotoxes[i].tox, &handle_friend_connection_status);
-        tox_callback_conference_invite(autotoxes[i].tox, &handle_conference_invite);
-        tox_callback_conference_connected(autotoxes[i].tox, &handle_conference_connected);
+        tox_events_callback_self_connection_status(autotoxes[i].dispatch, handle_self_connection_status);
+        tox_events_callback_friend_connection_status(autotoxes[i].dispatch, handle_friend_connection_status);
+        tox_events_callback_conference_invite(autotoxes[i].dispatch, handle_conference_invite);
+        tox_events_callback_conference_connected(autotoxes[i].dispatch, handle_conference_connected);
     }
 
     ck_assert_msg(toxav_add_av_groupchat(autotoxes[0].tox, audio_callback, &autotoxes[0]) != UINT32_MAX,
@@ -410,7 +426,6 @@ static void test_groupav(AutoTox *autotoxes)
     ck_assert_msg(tox_conference_invite(autotoxes[0].tox, 0, 0, nullptr) != 0, "failed to invite friend");
     ((State *)autotoxes[0].state)->invited_next = true;
 
-
     printf("waiting for invitations to be made\n");
     uint32_t invited_count = 0;
 
