@@ -13,24 +13,40 @@
 #include "auto_test_support.h"
 #include "check_compat.h"
 
-/* The Travis-CI container responds poorly to ::1 as a localhost address
- * You're encouraged to -D FORCE_TESTS_IPV6 on a local test  */
-#ifdef FORCE_TESTS_IPV6
+#ifndef USE_IPV6
+#define USE_IPV6 1
+#endif
+
+#ifdef TOX_LOCALHOST
+#undef TOX_LOCALHOST
+#endif
+#if USE_IPV6
 #define TOX_LOCALHOST "::1"
 #else
 #define TOX_LOCALHOST "127.0.0.1"
 #endif
 
+typedef struct State {
+    uint32_t to_comp;
+    Tox *tox;
+} State;
+
 static bool enable_broken_tests = false;
 
-static void accept_friend_request(Tox *m, const uint8_t *public_key, const uint8_t *data, size_t length, void *userdata)
+static void accept_friend_request(const Tox_Event_Friend_Request *event, void *userdata)
 {
-    if (*((uint32_t *)userdata) != 974536) {
+    State *state = (State *)userdata;
+
+    const uint8_t *public_key = tox_event_friend_request_get_public_key(event);
+    const uint8_t *message = tox_event_friend_request_get_message(event);
+    const uint32_t message_length = tox_event_friend_request_get_message_length(event);
+
+    if (state->to_comp != 974536) {
         return;
     }
 
-    if (length == 7 && memcmp("Gentoo", data, 7) == 0) {
-        tox_friend_add_norequest(m, public_key, nullptr);
+    if (message_length == 7 && memcmp("Gentoo", message, 7) == 0) {
+        tox_friend_add_norequest(state->tox, public_key, nullptr);
     }
 }
 
@@ -41,15 +57,14 @@ static uint16_t tcp_relay_port = 33448;
 
 static void test_many_clients_tcp(void)
 {
-    const Random *rng = system_random();
+    const Random *rng = os_random();
     ck_assert(rng != nullptr);
     long long unsigned int cur_time = time(nullptr);
     Tox *toxes[NUM_TOXES_TCP];
     uint32_t index[NUM_TOXES_TCP];
-    uint32_t i, j;
     uint32_t to_comp = 974536;
 
-    for (i = 0; i < NUM_TOXES_TCP; ++i) {
+    for (uint32_t i = 0; i < NUM_TOXES_TCP; ++i) {
         struct Tox_Options *opts = tox_options_new(nullptr);
 
         if (i == 0) {
@@ -69,10 +84,10 @@ static void test_many_clients_tcp(void)
             continue;
         }
         ck_assert_msg(toxes[i] != nullptr, "Failed to create tox instances %u", i);
-        tox_callback_friend_request(toxes[i], accept_friend_request);
+        tox_events_init(toxes[i]);
         uint8_t dpk[TOX_PUBLIC_KEY_SIZE];
         tox_self_get_dht_id(toxes[0], dpk);
-        Tox_Err_Bootstrap error = TOX_ERR_BOOTSTRAP_OK;
+        Tox_Err_Bootstrap error;
         ck_assert_msg(tox_add_tcp_relay(toxes[i], TOX_LOCALHOST, tcp_relay_port, dpk, &error), "add relay error, %u, %d", i,
                       error);
         uint16_t first_port = tox_self_get_udp_port(toxes[0], nullptr);
@@ -81,6 +96,11 @@ static void test_many_clients_tcp(void)
         tox_options_free(opts);
     }
 
+    Tox_Dispatch *dispatch = tox_dispatch_new(nullptr);
+    ck_assert(dispatch != nullptr);
+
+    tox_events_callback_friend_request(dispatch, accept_friend_request);
+
     struct {
         uint16_t tox1;
         uint16_t tox2;
@@ -88,12 +108,12 @@ static void test_many_clients_tcp(void)
 
     uint8_t address[TOX_ADDRESS_SIZE];
 
-    for (i = 0; i < NUM_FRIENDS; ++i) {
+    for (uint32_t i = 0; i < NUM_FRIENDS; ++i) {
 loop_top:
         pairs[i].tox1 = random_u32(rng) % NUM_TOXES_TCP;
         pairs[i].tox2 = (pairs[i].tox1 + random_u32(rng) % (NUM_TOXES_TCP - 1) + 1) % NUM_TOXES_TCP;
 
-        for (j = 0; j < i; ++j) {
+        for (uint32_t j = 0; j < i; ++j) {
             if (pairs[j].tox2 == pairs[i].tox1 && pairs[j].tox1 == pairs[i].tox2) {
                 goto loop_top;
             }
@@ -114,8 +134,8 @@ loop_top:
     while (true) {
         uint16_t counter = 0;
 
-        for (i = 0; i < NUM_TOXES_TCP; ++i) {
-            for (j = 0; j < tox_self_get_friend_list_size(toxes[i]); ++j) {
+        for (uint32_t i = 0; i < NUM_TOXES_TCP; ++i) {
+            for (uint32_t j = 0; j < tox_self_get_friend_list_size(toxes[i]); ++j) {
                 if (tox_friend_get_connection_status(toxes[i], j, nullptr) == TOX_CONNECTION_TCP) {
                     ++counter;
                 }
@@ -126,14 +146,20 @@ loop_top:
             break;
         }
 
-        for (i = 0; i < NUM_TOXES_TCP; ++i) {
-            tox_iterate(toxes[i], &to_comp);
+        for (uint32_t i = 0; i < NUM_TOXES_TCP; ++i) {
+            Tox_Err_Events_Iterate err = TOX_ERR_EVENTS_ITERATE_OK;
+            Tox_Events *events = tox_events_iterate(toxes[i], true, &err);
+            ck_assert(err == TOX_ERR_EVENTS_ITERATE_OK);
+            State state = {to_comp, toxes[i]};
+            tox_dispatch_invoke(dispatch, events, &state);
+            tox_events_free(events);
         }
 
         c_sleep(50);
     }
 
-    for (i = 0; i < NUM_TOXES_TCP; ++i) {
+    tox_dispatch_free(dispatch);
+    for (uint32_t i = 0; i < NUM_TOXES_TCP; ++i) {
         tox_kill(toxes[i]);
     }
 
@@ -144,15 +170,14 @@ loop_top:
 
 static void test_many_clients_tcp_b(void)
 {
-    const Random *rng = system_random();
+    const Random *rng = os_random();
     ck_assert(rng != nullptr);
     long long unsigned int cur_time = time(nullptr);
     Tox *toxes[NUM_TOXES_TCP];
     uint32_t index[NUM_TOXES_TCP];
-    uint32_t i, j;
     uint32_t to_comp = 974536;
 
-    for (i = 0; i < NUM_TOXES_TCP; ++i) {
+    for (uint32_t i = 0; i < NUM_TOXES_TCP; ++i) {
         struct Tox_Options *opts = tox_options_new(nullptr);
 
         if (i < NUM_TCP_RELAYS) {
@@ -164,7 +189,7 @@ static void test_many_clients_tcp_b(void)
         index[i] = i + 1;
         toxes[i] = tox_new_log(opts, nullptr, &index[i]);
         ck_assert_msg(toxes[i] != nullptr, "Failed to create tox instances %u", i);
-        tox_callback_friend_request(toxes[i], accept_friend_request);
+        tox_events_init(toxes[i]);
         uint8_t dpk[TOX_PUBLIC_KEY_SIZE];
         tox_self_get_dht_id(toxes[(i % NUM_TCP_RELAYS)], dpk);
         ck_assert_msg(tox_add_tcp_relay(toxes[i], TOX_LOCALHOST, tcp_relay_port + (i % NUM_TCP_RELAYS), dpk, nullptr),
@@ -176,6 +201,11 @@ static void test_many_clients_tcp_b(void)
         tox_options_free(opts);
     }
 
+    Tox_Dispatch *dispatch = tox_dispatch_new(nullptr);
+    ck_assert(dispatch != nullptr);
+
+    tox_events_callback_friend_request(dispatch, accept_friend_request);
+
     struct {
         uint16_t tox1;
         uint16_t tox2;
@@ -183,12 +213,12 @@ static void test_many_clients_tcp_b(void)
 
     uint8_t address[TOX_ADDRESS_SIZE];
 
-    for (i = 0; i < NUM_FRIENDS; ++i) {
+    for (uint32_t i = 0; i < NUM_FRIENDS; ++i) {
 loop_top:
         pairs[i].tox1 = random_u32(rng) % NUM_TOXES_TCP;
         pairs[i].tox2 = (pairs[i].tox1 + random_u32(rng) % (NUM_TOXES_TCP - 1) + 1) % NUM_TOXES_TCP;
 
-        for (j = 0; j < i; ++j) {
+        for (uint32_t j = 0; j < i; ++j) {
             if (pairs[j].tox2 == pairs[i].tox1 && pairs[j].tox1 == pairs[i].tox2) {
                 goto loop_top;
             }
@@ -211,8 +241,8 @@ loop_top:
     while (true) {
         uint16_t counter = 0;
 
-        for (i = 0; i < NUM_TOXES_TCP; ++i) {
-            for (j = 0; j < tox_self_get_friend_list_size(toxes[i]); ++j) {
+        for (uint32_t i = 0; i < NUM_TOXES_TCP; ++i) {
+            for (uint32_t j = 0; j < tox_self_get_friend_list_size(toxes[i]); ++j) {
                 if (tox_friend_get_connection_status(toxes[i], j, nullptr) == TOX_CONNECTION_TCP) {
                     ++counter;
                 }
@@ -228,21 +258,26 @@ loop_top:
             break;
         }
 
-        for (i = 0; i < NUM_TOXES_TCP; ++i) {
-            tox_iterate(toxes[i], &to_comp);
+        for (uint32_t i = 0; i < NUM_TOXES_TCP; ++i) {
+            Tox_Err_Events_Iterate err = TOX_ERR_EVENTS_ITERATE_OK;
+            Tox_Events *events = tox_events_iterate(toxes[i], true, &err);
+            ck_assert(err == TOX_ERR_EVENTS_ITERATE_OK);
+            State state = {to_comp, toxes[i]};
+            tox_dispatch_invoke(dispatch, events, &state);
+            tox_events_free(events);
         }
 
         c_sleep(30);
     }
 
-    for (i = 0; i < NUM_TOXES_TCP; ++i) {
+    tox_dispatch_free(dispatch);
+    for (uint32_t i = 0; i < NUM_TOXES_TCP; ++i) {
         tox_kill(toxes[i]);
     }
 
     printf("test_many_clients_tcp_b succeeded, took %llu seconds\n", time(nullptr) - cur_time);
 }
 
-
 static void tox_suite(void)
 {
     /* Each tox connects to a single tox TCP    */
