@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-3.0-or-later
- * Copyright © 2016-2018 The TokTok team.
+ * Copyright © 2016-2024 The TokTok team.
  * Copyright © 2014-2016 Tox project.
  */
 
@@ -19,19 +19,19 @@
  *            also, iff `tcp_relay_ports_count` > 0, then you are responsible for freeing `tcp_relay_ports`
  *            and also `motd` iff `enable_motd` is set.
  *
- * @return 1 on success,
- *         0 on failure, doesn't modify any data pointed by arguments.
+ * @return true on success,
+ *         false on failure, doesn't modify any data pointed by arguments.
  */
-int get_general_config(const char *cfg_file_path, char **pid_file_path, char **keys_file_path, int *port,
-                       int *enable_ipv6, int *enable_ipv4_fallback, int *enable_lan_discovery, int *enable_tcp_relay,
-                       uint16_t **tcp_relay_ports, int *tcp_relay_port_count, int *enable_motd, char **motd);
+bool get_general_config(const char *cfg_file_path, char **pid_file_path, char **keys_file_path, int *port,
+                        int *enable_ipv6, int *enable_ipv4_fallback, int *enable_lan_discovery, int *enable_tcp_relay,
+                        uint16_t **tcp_relay_ports, int *tcp_relay_port_count, int *enable_motd, char **motd);
 
 /**
  * Bootstraps off nodes listed in the config file.
  *
- * @return 1 on success, some or no bootstrap nodes were added
- *         0 on failure, a error accured while parsing config file.
+ * @return true on success, some or no bootstrap nodes were added
+ *         false on failure, an error occurred while parsing the config file.
  */
-int bootstrap_from_config(const char *cfg_file_path, DHT *dht, int enable_ipv6);
+bool bootstrap_from_config(const char *cfg_file_path, DHT *dht, bool enable_ipv6);
 
 #endif // C_TOXCORE_OTHER_BOOTSTRAP_DAEMON_SRC_CONFIG_H
