@@ -10,10 +10,12 @@
 #include "log_backend_syslog.h"
 
 #include "global.h"
+#include "log.h"
 
 #include "../../../toxcore/ccompat.h"
 
 #include <assert.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <syslog.h>
@@ -49,7 +51,7 @@ void log_backend_syslog_write(LOG_LEVEL level, const char *format, va_list args)
     va_list args2;
 
     va_copy(args2, args);
-    int size = vsnprintf(nullptr, 0, format, args2);
+    const int size = vsnprintf(nullptr, 0, format, args2);
     va_end(args2);
 
     assert(size >= 0);
@@ -59,6 +61,9 @@ void log_backend_syslog_write(LOG_LEVEL level, const char *format, va_list args)
     }
 
     char *buf = (char *)malloc(size + 1);
+    if (buf == nullptr) {
+        return;
+    }
     vsnprintf(buf, size + 1, format, args);
 
     syslog(log_backend_syslog_level(level), "%s", buf);
