@@ -8,14 +8,9 @@
 #include "../toxcore/Messenger.h"
 #include "../toxcore/tox.h"
 
-#ifndef TOX_DEFINED
-#define TOX_DEFINED
-typedef struct Tox Tox;
-#endif /* TOX_DEFINED */
+typedef struct BWController BWController;
 
-typedef struct BWController_s BWController;
-
-typedef void m_cb(BWController *bwc, uint32_t friend_number, float todo, void *user_data);
+typedef void m_cb(BWController *bwc, uint32_t friend_number, float loss, void *user_data);
 
 BWController *bwc_new(Messenger *m, Tox *tox, uint32_t friendnumber, m_cb *mcb, void *mcb_user_data,
                       Mono_Time *bwc_mono_time);
@@ -25,4 +20,4 @@ void bwc_kill(BWController *bwc);
 void bwc_add_lost(BWController *bwc, uint32_t bytes_lost);
 void bwc_add_recv(BWController *bwc, uint32_t recv_bytes);
 
-#endif // C_TOXCORE_TOXAV_BWCONTROLLER_H
+#endif /* C_TOXCORE_TOXAV_BWCONTROLLER_H */
