@@ -27,14 +27,12 @@ typedef enum MSIHeaderID {
     ID_CAPABILITIES,
 } MSIHeaderID;
 
-
 typedef enum MSIRequest {
     REQU_INIT,
     REQU_PUSH,
     REQU_POP,
 } MSIRequest;
 
-
 typedef struct MSIHeaderRequest {
     MSIRequest value;
     bool exists;
@@ -50,30 +48,27 @@ typedef struct MSIHeaderCapabilities {
     bool exists;
 } MSIHeaderCapabilities;
 
-
 typedef struct MSIMessage {
     MSIHeaderRequest      request;
     MSIHeaderError        error;
     MSIHeaderCapabilities capabilities;
 } MSIMessage;
 
-
 static void msg_init(MSIMessage *dest, MSIRequest request);
 static int msg_parse_in(const Logger *log, MSIMessage *dest, const uint8_t *data, uint16_t length);
-static uint8_t *msg_parse_header_out(MSIHeaderID id, uint8_t *dest, const void *value, uint8_t value_len,
+static uint8_t *msg_parse_header_out(MSIHeaderID id, uint8_t *dest, const uint8_t *value, uint8_t value_len,
                                      uint16_t *length);
-static int send_message(Messenger *m, uint32_t friend_number, const MSIMessage *msg);
-static int send_error(Messenger *m, uint32_t friend_number, MSIError error);
+static int send_message(const Messenger *m, uint32_t friend_number, const MSIMessage *msg);
+static int send_error(const Messenger *m, uint32_t friend_number, MSIError error);
 static bool invoke_callback(MSICall *call, MSICallbackID cb);
 static MSICall *get_call(MSISession *session, uint32_t friend_number);
 static MSICall *new_call(MSISession *session, uint32_t friend_number);
 static void kill_call(MSICall *call);
-static void on_peer_status(Messenger *m, uint32_t friend_number, uint8_t status, void *data);
+static void on_peer_status(Messenger *m, uint32_t friend_number, bool is_online, void *user_data);
 static void handle_init(MSICall *call, const MSIMessage *msg);
 static void handle_push(MSICall *call, const MSIMessage *msg);
 static void handle_pop(MSICall *call, const MSIMessage *msg);
-static void handle_msi_packet(Messenger *m, uint32_t friend_number, const uint8_t *data, uint16_t length, void *object);
-
+static void handle_msi_packet(Messenger *m, uint32_t friend_number, const uint8_t *data, uint16_t length, void *user_data);
 
 /*
  * Public functions
@@ -318,7 +313,6 @@ int msi_change_capabilities(MSICall *call, uint8_t capabilities)
     return 0;
 }
 
-
 /**
  * Private functions
  */
@@ -357,7 +351,6 @@ static bool check_enum_high(const Logger *log, const uint8_t *bytes, uint8_t enu
     return true;
 }
 
-
 static int msg_parse_in(const Logger *log, MSIMessage *dest, const uint8_t *data, uint16_t length)
 {
     /* Parse raw data received from socket into MSIMessage struct */
@@ -425,7 +418,7 @@ static int msg_parse_in(const Logger *log, MSIMessage *dest, const uint8_t *data
 
     return 0;
 }
-static uint8_t *msg_parse_header_out(MSIHeaderID id, uint8_t *dest, const void *value, uint8_t value_len,
+static uint8_t *msg_parse_header_out(MSIHeaderID id, uint8_t *dest, const uint8_t *value, uint8_t value_len,
                                      uint16_t *length)
 {
     /* Parse a single header for sending */
@@ -444,12 +437,12 @@ static uint8_t *msg_parse_header_out(MSIHeaderID id, uint8_t *dest, const void *
 
     return dest + value_len; /* Set to next position ready to be written */
 }
-static int send_message(Messenger *m, uint32_t friend_number, const MSIMessage *msg)
+static int send_message(const Messenger *m, uint32_t friend_number, const MSIMessage *msg)
 {
     /* Parse and send message */
     assert(m != nullptr);
 
-    uint8_t parsed [MSI_MAXMSG_SIZE];
+    uint8_t parsed[MSI_MAXMSG_SIZE];
 
     uint8_t *it = parsed;
     uint16_t size = 0;
@@ -489,7 +482,7 @@ static int send_message(Messenger *m, uint32_t friend_number, const MSIMessage *
 
     return -1;
 }
-static int send_error(Messenger *m, uint32_t friend_number, MSIError error)
+static int send_error(const Messenger *m, uint32_t friend_number, MSIError error)
 {
     /* Send error message */
     assert(m != nullptr);
@@ -587,7 +580,7 @@ static MSICall *new_call(MSISession *session, uint32_t friend_number)
         session->calls_tail = friend_number;
         session->calls_head = friend_number;
     } else if (session->calls_tail < friend_number) { /* Appending */
-        MSICall **tmp = (MSICall **)realloc(session->calls, sizeof(MSICall *) * (friend_number + 1));
+        MSICall **tmp = (MSICall **)realloc(session->calls, (friend_number + 1) * sizeof(MSICall *));
 
         if (tmp == nullptr) {
             free(rc);
@@ -655,14 +648,15 @@ CLEAR_CONTAINER:
     free(call);
     session->calls = nullptr;
 }
-static void on_peer_status(Messenger *m, uint32_t friend_number, uint8_t status, void *data)
+static void on_peer_status(Messenger *m, uint32_t friend_number, bool is_online, void *user_data)
 {
-    if (status != 0) {
+    MSISession *session = (MSISession *)user_data;
+
+    if (is_online) {
         // Friend is online.
         return;
     }
 
-    MSISession *session = (MSISession *)data;
     LOGGER_DEBUG(m->log, "Friend %d is now offline", friend_number);
 
     pthread_mutex_lock(session->mutex);
@@ -821,6 +815,7 @@ static void handle_pop(MSICall *call, const MSIMessage *msg)
         switch (call->state) {
             case MSI_CALL_INACTIVE: {
                 LOGGER_FATAL(call->session->messenger->log, "Handling what should be impossible case");
+                break;
             }
 
             case MSI_CALL_ACTIVE: {
@@ -848,11 +843,12 @@ static void handle_pop(MSICall *call, const MSIMessage *msg)
 
     kill_call(call);
 }
-static void handle_msi_packet(Messenger *m, uint32_t friend_number, const uint8_t *data, uint16_t length, void *object)
+static void handle_msi_packet(Messenger *m, uint32_t friend_number, const uint8_t *data, uint16_t length, void *user_data)
 {
+    MSISession *session = (MSISession *)user_data;
+
     LOGGER_DEBUG(m->log, "Got msi message");
 
-    MSISession *session = (MSISession *)object;
     MSIMessage msg;
 
     if (msg_parse_in(m->log, &msg, data, length) == -1) {
