@@ -52,7 +52,6 @@ typedef enum RTPFlags {
     RTP_KEY_FRAME = 1 << 1,
 } RTPFlags;
 
-
 struct RTPHeader {
     /* Standard RTP header */
     unsigned ve: 2; /* Version has only 2 bits! */
@@ -100,7 +99,6 @@ struct RTPHeader {
     uint16_t data_length_lower;
 };
 
-
 struct RTPMessage {
     /**
      * This is used in the old code that doesn't deal with large frames, i.e.
@@ -167,7 +165,6 @@ typedef struct RTPSession {
     rtp_m_cb *mcb;
 } RTPSession;
 
-
 /**
  * Serialise an RTPHeader to bytes to be sent over the network.
  *
@@ -204,7 +201,7 @@ int rtp_send_data(RTPSession *session, const uint8_t *data, uint32_t length,
                   bool is_keyframe, const Logger *log);
 
 #ifdef __cplusplus
-}  // extern "C"
+} /* extern "C" */
 #endif
 
-#endif // C_TOXCORE_TOXAV_RTP_H
+#endif /* C_TOXCORE_TOXAV_RTP_H */
