@@ -9,8 +9,13 @@
 #ifndef C_TOXCORE_TOXCORE_TCP_SERVER_H
 #define C_TOXCORE_TOXCORE_TCP_SERVER_H
 
+#include "attributes.h"
 #include "crypto_core.h"
 #include "forwarding.h"
+#include "logger.h"
+#include "mem.h"
+#include "mono_time.h"
+#include "network.h"
 #include "onion.h"
 
 #define MAX_INCOMING_CONNECTIONS 256
@@ -34,18 +39,17 @@ non_null()
 size_t tcp_server_listen_count(const TCP_Server *tcp_server);
 
 /** Create new TCP server instance. */
-non_null(1, 2, 3, 6, 7) nullable(8, 9)
-TCP_Server *new_TCP_server(const Logger *logger, const Random *rng, const Network *ns,
+non_null(1, 2, 3, 4, 7, 8) nullable(9, 10)
+TCP_Server *new_tcp_server(const Logger *logger, const Memory *mem, const Random *rng, const Network *ns,
                            bool ipv6_enabled, uint16_t num_sockets, const uint16_t *ports,
                            const uint8_t *secret_key, Onion *onion, Forwarding *forwarding);
 
 /** Run the TCP_server */
 non_null()
-void do_TCP_server(TCP_Server *tcp_server, const Mono_Time *mono_time);
+void do_tcp_server(TCP_Server *tcp_server, const Mono_Time *mono_time);
 
 /** Kill the TCP server */
 nullable(1)
-void kill_TCP_server(TCP_Server *tcp_server);
+void kill_tcp_server(TCP_Server *tcp_server);
 
-
-#endif
+#endif /* C_TOXCORE_TOXCORE_TCP_SERVER_H */
