@@ -19,7 +19,7 @@ extern "C" {
 
 #ifndef MIN_LOGGER_LEVEL
 #define MIN_LOGGER_LEVEL LOGGER_LEVEL_INFO
-#endif
+#endif /* MIN_LOGGER_LEVEL */
 
 // NOTE: Don't forget to update build system files after modifying the enum.
 typedef enum Logger_Level {
@@ -67,6 +67,8 @@ void logger_write(
     const Logger *log, Logger_Level level, const char *file, int line, const char *func,
     const char *format, ...);
 
+/* @brief Terminate the program with a signal. */
+void logger_abort(void);
 
 #define LOGGER_WRITE(log, level, ...)                                            \
     do {                                                                         \
@@ -85,7 +87,7 @@ void logger_write(
 #define LOGGER_FATAL(log, ...)          \
     do {                                \
         LOGGER_ERROR(log, __VA_ARGS__); \
-        abort();                        \
+        logger_abort();                 \
     } while (0)
 
 #define LOGGER_ASSERT(log, cond, ...)              \
@@ -97,7 +99,7 @@ void logger_write(
     } while (0)
 
 #ifdef __cplusplus
-}  // extern "C"
+} /* extern "C" */
 #endif
 
-#endif // C_TOXCORE_TOXCORE_LOGGER_H
+#endif /* C_TOXCORE_TOXCORE_LOGGER_H */
